import { ZodFirstPartyTypeKind } from "zod/v3";
import { Refs } from "./Refs.js";
import { JsonSchema7Type } from "./parseTypes.js";
export type InnerDefGetter = () => any;
export declare const selectParser: (def: any, typeName: ZodFirstPartyTypeKind, refs: Refs) => JsonSchema7Type | undefined | InnerDefGetter;
